<?php
/**
* © Copyright 2007 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
*  iHRIS_Person
* @package I2CE
* @subpackage Core
* @author Carl Leitner <litlfred@ibiblio.org>
* @copyright Copyright &copy; 2007 IntraHealth International, Inc. 
* This file is part of I2CE. I2CE is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free Software Foundation; either 
* version 3 of the License, or (at your option) any later version. I2CE is distributed in the hope 
* that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
* or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have 
* received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
* @version 2.1
* @access public
*/


class iHRIS_Person extends I2CE_Form{


    /**
     * Create a new instance of a person.
     * 
     * @param string $form The name of this form
     * @param integer $id
     */
    public function __construct( $form, $id = 0 ) {                
        parent::__construct( $form, $id );
        $this->ignore_error = false;
    }


    /**
     * Perform additional validation for the Person object.
     * 
     * A new person record needs to verify there aren't any existing records with the same name.
     * @param I2CE_Template &$page A reference to the top level Template object
     */
    public function validate( &$page ) {
        parent::validate( $page );
        // Search for existing records with the same name.
        $search = array();
        if ( $this->id == 0 && !$this->surname_ignore && I2CE_Validate::checkString( $this->surname ) && I2CE_Validate::checkString( $this->firstname ) ) {
            $search["person"][] = array( 'field' => 'surname', 'history' => true,
                                         'values' => array( 0 => array( 'value' => strtolower( $this->surname ), 
                                                                        'method' => 'LIKE', 'lower' => true ) ) );
            $search["person"][] = array( 'field' => 'firstname', 'history' => true,
                                         'values' => array( 0 => array( 'value' => strtolower( $this->firstname ), 
                                                                        'method' => 'LIKE', 'lower' => true ) ) );
            $results = self::search( $search );
            if( count($results) > 0 ) {
                $page->setFormInvalid( $this->getField('surname')->getHTMLName(), "Duplicate records match this record's name:",
                                       array( "view?id=" => $results ) );
            }
        }
        
    }



    /**
     * Search the database for a person record.
     * 
     * Returns a list of records that match the given search parameters.
     * The $search parameter is an array in the following format:
     * array( 
     *  '<form>' => array( 
     *          0 => array( 'field' => '<field_name>', 
     *                  'values' => array( 0 => array( 'value' => '<field_value>', 'method' => '=', 'lower' => <boolean> ) ),
     *                  - or -
     *                  'values' => 'norecord',
     *                  'value_andor' => 'OR',
     *                  'history' => <false|true>
     * )
     * 
     * $andor can be an array.  If it is, the first entry is the default value used.  The 1st entry goes between the 1st two where clauses, etc.
     * 
     * Any forms not matching person will use the parent id from the record entry for the matching results.
     * @param array $search
     * @param string $andor If the search results should be AND or OR among all the search criteria
     * @param boolean $modified
     * @param array $limit The limit to use on the final query, ie: array( 0, 100 ) to start at the beginning
     *                     and return 100 rows.
     * @return array
     */
    static public function search( $search, $andor = "AND", $modified=false, $limit=null ) {
        $db = MDB2::singleton();
        I2CE::pearError( $db, "Error getting database connection: " );

        $fname_details = I2CE_FormField::getFormFieldIdAndType( "person", "firstname" );
        $surname_details = I2CE_FormField::getFormFieldIdAndType( "person", "surname" );
        $othername_details = I2CE_FormField::getFormFieldIdAndType( "person", "othername" );
        if ( !$fname_details || !$surname_details || !$othername_details ) {
            return array();
        }

        $params = array();
                                
        $modified_query = "";
        if ( $modified !== false ) {
            if ( $modified === true ) {
                $modified_query = ",(SELECT MAX(last_modified) FROM record WHERE parent = e1.record OR id = e1.record) AS last_modified";
            } else {
                $mod_form_id = I2CE_Form::getFormId( $modified, true );
                if ( $mod_form_id > 0 ) {
                    $modified_query = ",(SELECT MAX(last_modified) FROM record WHERE (parent = e1.record OR id = e1.record) AND form = ?) AS last_modified";
                    $params[] = $mod_form_id;
                }
            }
        }
        $query = "SELECT SQL_CALC_FOUND_ROWS e1.record,CONCAT(e1." . $surname_details['type'] . "_value, ', ', e2." 
            . $fname_details['type'] . "_value, ' ', IFNULL(e3." . $othername_details['type'] . "_value, '') ) AS name"
            . $modified_query
            . " FROM last_entry e1 JOIN last_entry e2 ON e1.record = e2.record JOIN last_entry e3 ON e1.record = e3.record WHERE e1.form_field = ? AND e2.form_field = ? AND e3.form_field = ?";
        $params[] = $surname_details['id'];
        $params[] = $fname_details['id'];
        $params[] = $othername_details['id'];
        $where = array();
        foreach( $search as $form => $form_searches ) {
            foreach(  $form_searches as $form_search ) {
                $details = I2CE_FormField::getFormFieldIdAndType( $form, $form_search['field'] );
                if ( $details == null ) continue;

                if ( array_key_exists( 'value_andor', $form_search ) ) {
                    $value_andor = $form_search['value_andor'];
                } else {
                    $value_andor = 'OR';
                }
                $add_where = "( SUBWHERE )";
                $sub_where = array();
                if ( array_key_exists('history', $form_search) && $form_search['history'] == true ) {
                    $table = "entry";
                } else {
                    $table = "last_entry";
                }
                if ( array_key_exists( 'not', $form_search ) && $form_search['not']) {
                	$not = " NOT ";
                } else {
                	$not = "";
                }
                if ( $form == "person" ) {
                    if ( $form_search['field'] == "surname" ) {
                        //$search_field = "e1." . $details['type'] . "_value";
                        // Search all entries for surname instead of just the current one.
                        $search_field = $details['type'] . "_value";
                        if ( $table == "entry" ) {
                            $add_where = "e1.record $not IN ( SELECT record FROM entry WHERE form_field = ? AND ( SUBWHERE ) )";
                            $params[] = $details['id'];
                        } else {
                            $search_field = "e1." . $details['type'] . "_value";
                        }
                    } elseif ( $form_search['field'] == "firstname" ) {
                        if ( $table == "entry" ) {
                            $add_where = "e1.record $not IN ( SELECT record FROM entry WHERE form_field = ? AND ( SUBWHERE ) )";
                            $params[] = $details['id'];                                                         
                        } else {
                            $search_field = "e2." . $details['type'] . "_value";
                        }
                    } else {
                        $search_field = $details['type'] . "_value";
                        $add_where = "e1.record $not IN ( SELECT record FROM " . $table . " WHERE form_field = ? AND ( SUBWHERE ) )";
                        $params[] = $details['id'];
                    }
                } elseif ( is_array( $form_search['values'] ) ) {
                    $search_field = "e." . $details['type'] . "_value";
                    if ( array_key_exists( "parent", $form_search ) && $form_search['parent'] ) {
                        $add_where = "e1.record $not IN ( SELECT pr.parent FROM record pr WHERE pr.id IN ( SELECT r.parent FROM record r JOIN " . $table . " e ON e.record = r.id WHERE e.form_field = ? AND ( SUBWHERE ) ) )";
                    } else {
                        $add_where = "e1.record $not IN ( SELECT r.parent FROM record r JOIN " . $table . " e ON e.record = r.id WHERE e.form_field = ? AND ( SUBWHERE ) )";
                    }
                    $params[] = $details['id'];
                } else {
                    $where[] = "e1.record NOT IN ( SELECT parent FROM record WHERE form = ? )";
                    $params[] = I2CE_Form::getFormId( $form );
                }
                if ( is_array( $form_search['values'] ) ) {
                    foreach( $form_search['values'] as $value ) {
                        if ( array_key_exists( 'method', $value ) ) {
                            $method = $value['method'];
                        } else {
                            $method = "=";
                        }
                        if ( $method == "ISNULL" ) {
                            $sub_where[] = $search_field . " IS NULL";
                            continue;
                        }
                        if ( array_key_exists( 'lower', $value ) ) {
                            $lower = $value['lower'];
                        } else {
                            $lower = false;
                        }
                        if ( $value['value'] == '%%' ) continue;
                        $sub_where[] = ($lower ? "LOWER(" : "" ) . $search_field . ($lower ? ")" : "") . " " . $method . " ?";
                        $params[] = $value['value'];
                    }
                    if ( count( $sub_where ) > 0 ) {
                        $where[] = str_replace( "SUBWHERE", implode( " " . $value_andor . " ", $sub_where ), $add_where );
                    }
                }
            }
        }
        
        if ( count( $where ) > 0 ) {
            if ( is_array( $andor ) && count($andor) > 1 ) {
                $where_clause = "";
                for( $i = 1; $i < count( $andor ); $i++ ) {
                    if ( $where_clause == "" ) {
                        $where_clause = "( " . $where[$i-1] . " " . $andor[$i] . " " . $where[$i] . " )";
                    } else {
                        $where_clause = "( " . $where_clause . " " . $andor[$i] . " " . $where[$i] . " )";
                    }
                }
                if ( count($where) > count($andor) ) {
                    $where_clause = $where_clause . " " . $andor[0] . " (" . implode( " " . $andor[0]. " ", array_slice( $where, count($andor) ) ) . " )";
                }
                $query .= " AND ( " . $where_clause . " )"; 
            } else {
                if ( is_array( $andor ) ) $andor = $andor[0];
                $query .= " AND ( " . implode( " " . $andor . " ", $where ) . " )";
            }
        }
        $query .= " ORDER BY name";
        if ( is_array( $limit ) && count($limit) > 0 && count($limit) < 3 ) {
            $query .= " LIMIT " . implode( ',', $limit );
        }
        $search_results = array();
        $sth = $db->prepare( $query, null, MDB2_PREPARE_RESULT );
        if ( !I2CE::pearError( $sth, "Error preparing person search: " ) ) {
            $results = $sth->execute( $params );
            if ( !I2CE::pearError( $results, "Error executing person search: " ) ) {
                if ( is_array( $limit ) ) {
                    $num_rows = $db->queryRow( "SELECT FOUND_ROWS() AS num_rows" );
                    if ( !I2CE::pearError( $num_rows, "Error counting person search results: " ) ) {
                        $search_results['count'] = $num_rows->num_rows;
                    } else {
                        $search_results['count'] = "?";
                    }
                }
                while ( $data = $results->fetchRow() ) {
                    $search_results[ $data->record ] = ( $modified !== false ? array( 'name' => $data->name, 'last_modified' => $data->last_modified ) : $data->name );
                }
                $results->free();
            }
            $sth->free();
        }
        return $search_results;
    }



}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
